# coding=utf-8

# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import subprocess

from osbuild import command
from osbuild import distro
from osbuild.plugins import interfaces


class PackageManager(interfaces.PackageManager):
    def __init__(self, test=False, interactive=True):
        import apt

        self._test = test
        self._interactive = interactive

        self._cache = apt.cache.Cache()

    def install_packages(self, packages):
        args = ["apt-get", "--no-install-recommends"]

        if not self._interactive:
            args.append("-y")

        args.append("install")
        args.extend(packages)

        command.run_with_sudo(args, test=self._test,
                              interactive=self._interactive)

    def remove_packages(self, packages):
        args = ["dpkg", "-P"]
        args.extend(packages)

        command.run_with_sudo(args, test=self._test)

    def update(self):
        command.run_with_sudo(["apt-get", "update"], test=self._test)

        args = ["apt-get"]

        if not self._interactive:
            args.append("-y")

        args.append("upgrade")

        command.run_with_sudo(args, test=self._test,
                              interactive=self._interactive)

    def find_all(self):
        return [package.name for package in self._cache
                if package.installed is not None]

    def _find_deps(self, package, result):
        if self._cache.is_virtual_package(package):
            for providing in self._cache.get_providing_packages(package):
                self._find_deps(providing.name, result)
            return

        if package not in self._cache:
            print("Package %s not in cache" % package)
            return

        installed = self._cache[package].installed
        if installed is None:
            print("Package %s not installed" % package)
            return

        for dependency in installed.dependencies:
            for base_dependency in dependency.or_dependencies:
                dependency_name = base_dependency.name
                if dependency_name not in result:
                    result.append(dependency_name)
                    self._find_deps(dependency_name, result)

    def find_with_deps(self, package_names):
        result = []

        for package in package_names:
            if package is not None:
                self._find_deps(package, result)
                if package not in result:
                    result.append(package)

        return result


distro.register_package_manager("debian", PackageManager)


class DistroInfo(interfaces.DistroInfo):
    _DEBIAN_VERSION_PATH = "/etc/debian_version"

    def __init__(self):
        arch = self._get_architecture()

        self.name = "debian"
        self.version = "unknown"
        self.gnome_version = "3.4"
        self.valid = True
        self.supported = (arch in ["i686", "x86_64"])
        self.lib_dir = None

        if arch == "i686":
            self.lib_dir = "lib/i386-linux-gnu"
        elif arch == "x86_64":
            self.lib_dir = "lib/x86_64-linux-gnu"

        try:
            with open(self._DEBIAN_VERSION_PATH) as f:
                debian_version = f.read().strip()
        except IOError:
            debian_version = None

        if debian_version is None:
            self.valid = False

        if debian_version and debian_version == "jessie/sid":
            self.version = "jessie"
        else:
            self.supported = False

    def _get_architecture(self):
        return subprocess.check_output(["arch"]).strip()

distro.register_distro_info(DistroInfo)



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os

from osbuild import distro
from osbuild import utils

config_dir = None
docs_dir = None
logs_dir = None
install_dir = None
lib_dir = None
share_dir = None
bin_dir = None
etc_dir = None
libexec_dir = None
package_files = None
system_lib_dirs = None
home_dir = None
build_state_dir = None
log_path = None
git_user_name = None
git_email = None

_source_dir = None
_prefs_path = None


class Module:
    def __init__(self, info):
        self.name = info["name"]
        self.repo = info["repo"]
        self.branch = info.get("branch", "master")
        self.tag = info.get("tag", None)
        self.auto_install = info.get("auto-install", False)
        self.options = info.get("options", [])
        self.options_evaluated = info.get("options_evaluated", [])
        self.has_checks = info.get("has_checks", False)
        self.no_libdir = info.get("no_libdir", False)
        self.makefile_name = info.get("makefile_name", "Makefile")
        self.has_docs = info.get("has_docs", False)
        self.docs_dir = info.get("docs_dir", self.name)

    def get_source_dir(self):
        return os.path.join(get_source_dir(), self.name)

    def get_build_system(self):
        source_dir = self.get_source_dir()

        package_json = os.path.join(source_dir, "package.json")

        if os.path.exists(package_json):
            with open(package_json) as f:
                parsed_json = json.load(f)
                if "volo" in parsed_json:
                    return "volo"
                else:
                    return "npm"
        elif os.path.exists(os.path.join(source_dir, "setup.py")):
            return "distutils"
        elif (os.path.exists(os.path.join(source_dir, "autogen.sh")) or
              os.path.exists(os.path.join(source_dir, "configure"))):
            return "autotools"
        else:
            return None

def setup(**kwargs):
    global config_dir
    config_dir = kwargs.get("config_dir", None)

    global docs_dir
    docs_dir = kwargs["docs_dir"]

    global logs_dir
    logs_dir = kwargs["logs_dir"]
    utils.ensure_dir(logs_dir)

    global _prefs_path
    _prefs_path = kwargs.get("prefs_path", None)

    global _source_dir
    _source_dir = kwargs["source_dir"]

    _setup_state_dir(kwargs["state_dir"])
    _setup_install_dir(kwargs["install_dir"])

    global log_path
    if "log_name" in kwargs:
        log_path = _create_log(kwargs["log_name"])

    if "git_user_name" in kwargs:
        global git_user_name
        git_user_name = kwargs["git_user_name"]

    if "git_email" in kwargs:
        global git_email
        git_email = kwargs["git_email"]


def get_source_dir():
    global _source_dir
    utils.ensure_dir(_source_dir)
    return _source_dir


def get_pref(name):
    defaults = {"PROFILE": "default"}

    prefs = _read_prefs()
    return prefs.get(name, defaults.get(name, None))


def set_pref(name, value):
    prefs = _read_prefs()
    prefs[name] = value
    _save_prefs(prefs)


def get_full_build():
    with open(os.path.join(config_dir, "config.json")) as f:
        config = json.load(f)

    return config["full_build"]


def load_packages():
    with open(os.path.join(config_dir, "packages.json")) as f:
        return json.load(f)


def load_prerequisites():
    with open(os.path.join(config_dir, "prerequisites.json")) as f:
        return json.load(f)


def load_dependencies(should_filter=True):
    with open(os.path.join(config_dir, "dependencies.json")) as f:
        info = json.load(f)

        if should_filter:
            return filter(_filter_if, info)
        else:
            return info


def load_modules():
    with open(os.path.join(config_dir, "modules.json")) as f:
        return [Module(info) for info in filter(_filter_if, json.load(f))]


def check():
    dep_names = []
    for dep_info in load_dependencies(should_filter=False):
        dep_names.append(dep_info["name"])

    for dep_info in load_prerequisites():
        dep_names.append(dep_info["name"])

    missing_packages = dep_names[:]
    unused_packages = []

    for name in load_packages().keys():
        if name == "base-system":
            continue

        if name in missing_packages:
            missing_packages.remove(name)
        else:
            unused_packages.append(name)

    return {"missing_packages": missing_packages,
            "unused_packages": unused_packages}


def _create_log(prefix):
    logfile_path = None
    number = 0

    while logfile_path is None:
        name = "%s-%d.log" % (prefix, number)
        path = os.path.join(logs_dir, name)

        if not os.path.exists(path):
            logfile_path = path

        number = number + 1

    link_path = os.path.join(logs_dir, "%s.log" % prefix)

    try:
        os.unlink(link_path)
    except OSError:
        pass

    os.symlink(logfile_path, link_path)

    return logfile_path


def _filter_if(item):
    if "if" not in item:
        return True

    distro_info = distro.get_distro_info()
    globals = {"gnome_version": distro_info.gnome_version,
               "distro": "%s-%s" % (distro_info.name, distro_info.version)}

    return eval(item["if"], globals)


def _setup_state_dir(state_dir):
    utils.ensure_dir(state_dir)

    global build_state_dir
    build_state_dir = os.path.join(state_dir, "build")
    utils.ensure_dir(build_state_dir)

    base_home_dir = os.path.join(state_dir, "home")
    utils.ensure_dir(base_home_dir)

    global home_dir
    home_dir = os.path.join(base_home_dir, get_pref("PROFILE"))
    utils.ensure_dir(home_dir)


def _setup_install_dir(dir, relocatable=False):
    global system_lib_dirs
    global install_dir
    global share_dir
    global bin_dir
    global etc_dir
    global lib_dir
    global libexec_dir

    install_dir = dir
    utils.ensure_dir(install_dir)

    share_dir = os.path.join(install_dir, "share")
    bin_dir = os.path.join(install_dir, "bin")
    etc_dir = os.path.join(install_dir, "etc")
    libexec_dir = os.path.join(install_dir, "libexec")

    distro_info = distro.get_distro_info()

    relative_lib_dir = distro_info.lib_dir
    if relative_lib_dir is None:
        relative_lib_dir = "lib"

    lib_dir = os.path.join(install_dir, relative_lib_dir)

    system_lib_dirs = ["/usr/lib"]
    if distro_info.lib_dir is not None:
        system_lib_dirs.append(os.path.join("/usr", distro_info.lib_dir))


def _read_prefs():
    global _prefs_path

    if _prefs_path is None or not os.path.exists(_prefs_path):
        return {}

    prefs = {}
    with open(_prefs_path) as f:
        for line in f.readlines():
            splitted = line.strip().split("=")
            if len(splitted) == 2:
                prefs[splitted[0]] = splitted[1]

    return prefs


def _save_prefs(prefs):
    global _prefs_path

    if _prefs_path is None:
        return

    with open(_prefs_path, "w") as f:
        for pref in prefs.items():
            f.write("%s\n" % "=".join(pref))



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from osbuild import build
from osbuild import logs
from osbuild import state
from osbuild import docs


def clean(build_only=False, new_files=False):
    print("\n= Clean =\n")

    build.clean(new_files=new_files)
    docs.clean()
    state.clean(build_only=build_only)

    if not build_only:
        logs.clean()



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import subprocess
import time

from osbuild import config


def run(args, test=False, interactive=False, retry=0):
    if test:
        print(" ".join(args))
        return

    log_file = None
    subprocess_args = {"args": args}

    if config.log_path and not interactive:
        log_file = open(config.log_path, "a")
        subprocess_args["stdout"] = log_file
        subprocess_args["stderr"] = subprocess.STDOUT

    tries = 0
    while tries < retry + 1:
        try:
            tries = tries + 1
            subprocess.check_call(**subprocess_args)
            break
        except subprocess.CalledProcessError as e:
            print("\nCommand failed, tail of %s\n" % config.log_path)
            if config.log_path:
                subprocess.call(["tail", config.log_path])

            if tries < retry + 1:
                print("Retrying (attempt %d) in 1 minute" % tries)
                time.sleep(60)
            else:
                raise e

    if log_file:
        log_file.close()


def run_with_sudo(args, test=False, interactive=False, retry=0):
    args_with_sudo = ["sudo"]
    args_with_sudo.extend(args)

    print(" ".join(args_with_sudo))

    run(args_with_sudo, test=test, retry=retry, interactive=interactive)



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import fnmatch
import os
import multiprocessing
import shutil
import subprocess

from osbuild import command
from osbuild import config
from osbuild import state
from osbuild import utils
from osbuild import git

_builders = {}
_distributors = {}


def build_one(module_name):
    for module in config.load_modules():
        if module.name == module_name:
            return _build_module(module)

    return False


def pull_one(module_name):
    for module in config.load_modules():
        if module.name == module_name:
            return _pull_module(module)

    return False


def pull(revisions={}, lazy=False):
    to_pull = []
    for module in config.load_modules():
        git_module = git.get_module(module)
        if not lazy or not os.path.exists(git_module.local):
            to_pull.append(module)

    if to_pull:
        print("\n= Pulling =\n")

    for module in to_pull:
        revision = revisions.get(module.name, None)
        if not _pull_module(module, revision):
            return False

    return True


def build(full=False):
    to_build = []
    for module in config.load_modules():
        if not state.built_module_is_unchanged(module):
            to_build.append(module)

    if not to_build:
        return True

    print("\n= Building =\n")

    _ccache_reset()

    for module in to_build:
        if not _build_module(module):
            return False

    _ccache_print_stats()

    return True


def clean(new_files=False):
    print("* Emptying install directory")
    _empty_dir(config.install_dir)

    for module in config.load_modules():
        print("* Cleaning %s" % module.name)

        git_module = git.get_module(module)
        git_module.clean(new_files=new_files)


def _ccache_reset():
    subprocess.check_call(["ccache", "-z"], stdout=utils.devnull)


def _ccache_print_stats():
    print("\n= ccache statistics =\n")
    subprocess.check_call(["ccache", "-s"])


def _unlink_libtool_files():
    def func(arg, dirname, fnames):
        for fname in fnmatch.filter(fnames, "*.la"):
            os.unlink(os.path.join(dirname, fname))

    os.path.walk(config.lib_dir, func, None)


def _pull_module(module, revision=None):
    print("* Pulling %s" % module.name)

    git_module = git.get_module(module)

    try:
        git_module.update(revision)
    except subprocess.CalledProcessError:
        return False

    return True


def _eval_option(option):
    return eval(option, {"prefix": config.install_dir})


def _build_autotools(module, log):
    # Workaround for aclocal 1.11 (fixed in 1.12)
    aclocal_path = os.path.join(config.share_dir, "aclocal")
    utils.ensure_dir(aclocal_path)

    makefile_path = os.path.join(module.get_source_dir(), module.makefile_name)

    if not os.path.exists(makefile_path):
        configure = os.path.join(module.get_source_dir(), "autogen.sh")
        if not os.path.exists(configure):
            configure = os.path.join(module.get_source_dir(), "configure")

        args = [configure, "--prefix", config.install_dir]

        if not module.no_libdir:
            args.extend(["--libdir", config.lib_dir])

        args.extend(module.options)

        for option in module.options_evaluated:
            args.append(_eval_option(option))

        command.run(args, log)

    jobs = multiprocessing.cpu_count() * 2

    command.run(["make", "-j", "%d" % jobs], log)
    command.run(["make", "install"], log)

    _unlink_libtool_files()

_builders["autotools"] = _build_autotools


def _build_distutils(module, log):
    setup = os.path.join(module.get_source_dir(), "setup.py")
    command.run(["python", setup, "install", "--prefix",
                 config.install_dir], log)

_builders["distutils"] = _build_distutils


def _build_volo(module, log):
    pass

_builders["volo"] = _build_volo


def _build_npm(module, log):
    if os.path.exists(os.path.join(module.get_source_dir(),
                      "Gruntfile.coffee")):
        command.run(["npm", "install", "grunt"])
        command.run(["grunt", "build"])

    command.run(["npm", "install", "-g", "--prefix", config.install_dir], log)

_builders["npm"] = _build_npm


def _build_module(module, log=None):
    print("* Building %s" % module.name)

    source_dir = module.get_source_dir()

    if not os.path.exists(source_dir):
        print("Source directory does not exist. Please pull the sources "
              "before building.")
        return False

    os.chdir(source_dir)

    try:
        build_system = module.get_build_system()
        if build_system:
            _builders[build_system](module, log)
    except subprocess.CalledProcessError:
        return False

    state.built_module_touch(module)

    return True


def _empty_dir(dir_path):
    shutil.rmtree(dir_path, ignore_errors=True)
    os.mkdir(dir_path)



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess

from osbuild import utils


def start():
    xvfb_display = _find_free_display()
    xvfb_proc = subprocess.Popen(args=["Xvfb", xvfb_display],
                                 stdout=utils.devnull,
                                 stderr=subprocess.STDOUT)
    orig_display = os.environ.get("DISPLAY", None)
    os.environ["DISPLAY"] = xvfb_display

    return (xvfb_proc, orig_display)


def stop(xvfb_proc, orig_display):
    if orig_display is not None:
        os.environ["DISPLAY"] = orig_display

    xvfb_proc.terminate()


def _find_free_display():
    for i in range(100, 1000):
        display = ":%s" % i
        result = subprocess.call(args=["xdpyinfo", "--display", display],
                                 stdout=utils.devnull,
                                 stderr=subprocess.STDOUT)
        if result > 0:
            return display



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import json

from osbuild import config
from osbuild import sourcestamp

_BUILT_MODULES = "builtmodules"
_FULL_BUILD = "fullbuild"
_SYSTEM_CHECK = "syscheck"


def built_module_touch(module):
    built_modules = _load_state(_BUILT_MODULES, {})

    source_stamp = sourcestamp.compute(module.get_source_dir())
    built_modules[module.name] = {"source_stamp": source_stamp}

    _save_state(_BUILT_MODULES, built_modules)


def built_module_is_unchanged(module):
    built_modules = _load_state(_BUILT_MODULES, {})
    if module.name not in built_modules:
        return False

    built_module = built_modules[module.name]
    if "source_stamp" not in built_module:
        return False

    old_source_stamp = built_module["source_stamp"]
    new_source_stamp = sourcestamp.compute(module.get_source_dir())

    return old_source_stamp == new_source_stamp


def system_check_is_unchanged():
    system_check = _load_state(_SYSTEM_CHECK)
    if not system_check or not "config_stamp" in system_check:
        return False

    config_stamp = sourcestamp.compute(config.config_dir)

    return system_check["config_stamp"] == config_stamp


def system_check_touch():
    system_check = _load_state(_SYSTEM_CHECK, {})

    config_stamp = sourcestamp.compute(config.config_dir)
    system_check["config_stamp"] = config_stamp

    _save_state(_SYSTEM_CHECK, system_check)


def full_build_is_required():
    full_build = _load_state(_FULL_BUILD)
    if not full_build:
        return False

    return not (full_build["last"] == config.get_full_build())


def full_build_touch():
    full_build = _load_state(_FULL_BUILD, {})
    full_build["last"] = config.get_full_build()
    _save_state(_FULL_BUILD, full_build)


def clean(build_only=False):
    print("* Deleting state")

    names = [_BUILT_MODULES, _FULL_BUILD]
    if not build_only:
        names.append(_SYSTEM_CHECK)

    for name in names:
        try:
            os.unlink(_get_state_path(name))
        except OSError:
            pass


def _get_state_path(name):
    return os.path.join(config.build_state_dir, "%s.json" % name)


def _load_state(name, default=None):
    state = default

    try:
        with open(_get_state_path(name)) as f:
            state = json.load(f)
    except IOError:
        pass

    return state


def _save_state(name, state):
    with open(_get_state_path(name), "w+") as f:
        json.dump(state, f, indent=4)
        f.write('\n')



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pkgutil
import imp

from osbuild import config
from osbuild import environ
from osbuild import plugins
from osbuild import system
from osbuild import build
from osbuild import state
from osbuild import clean


def run_build(full=False):
    if full or state.full_build_is_required():
        clean.clean(build_only=True)
        environ.setup_gconf()

    state.full_build_touch()

    if not build.pull(lazy=True):
        return False

    if not build.build(full=False):
        return False

    return True


def load_plugins():
    for loader, name, ispkg in pkgutil.iter_modules(plugins.__path__):
        f, filename, desc = imp.find_module(name, plugins.__path__)
        imp.load_module(name, f, filename, desc)


def setup(config_args, check_args):
    load_plugins()

    config.setup(**config_args)

    if not system.check(**check_args):
        return False

    environ.setup_variables()
    environ.setup_gconf()

    return True



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess

from osbuild import config
from osbuild import command
from osbuild import xvfb
from osbuild import build

_checkers = {}


def check_one(module_name):
    for module in config.load_modules():
        if module.name == module_name:
            return _check_module(module)

    return False


def check():
    if not build.build():
        return False

    modules = config.load_modules()
    for module in modules:
        if not _check_module(module):
            return False

    return True


def _check_module(module):
    build_system = module.get_build_system()

    if build_system and module.has_checks:
        print("* Checking %s" % module.name)
        return _checkers[build_system](module)

    return True


def _volo_checker(module):
    orig_root = module.get_source_dir()
    for root, dirs, files in os.walk(module.get_source_dir()):
        if root == orig_root and "lib" in dirs:
            dirs.remove("lib")
        for f in files:
            if f.endswith(".js"):
                try:
                    command.run(["jshint", os.path.join(root, f)])
                except subprocess.CalledProcessError:
                    return False
    return True

_checkers['volo'] = _volo_checker


def _autotools_checker(module):
    result = True
    os.chdir(module.get_source_dir())
    xvfb_proc, orig_display = xvfb.start()

    try:
        command.run(["dbus-launch", "--exit-with-session",
                     "make", "check"])
    except subprocess.CalledProcessError:
        result = False

    xvfb.stop(xvfb_proc, orig_display)

    return result

_checkers['autotools'] = _autotools_checker



# coding=utf-8
# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import subprocess

from osbuild import command
from osbuild import distro
from osbuild.plugins import interfaces


class PackageManager(interfaces.PackageManager):
    def __init__(self, test=False, interactive=True):
        self._test = test
        self._interactive = interactive

    def install_packages(self, packages):
        args = ["yum"]

        if not self._interactive:
            args.append("-y")

        args.append("install")
        args.extend(packages)

        command.run_with_sudo(args, test=self._test,
                              interactive=self._interactive)

    def remove_packages(self, packages):
        args = ["rpm", "-e"]
        args.extend(packages)

        command.run_with_sudo(args, test=self._test)

    def update(self):
        args = ["yum"]

        if not self._interactive:
            args.append("-y")

        args.append("update")

        command.run_with_sudo(args, test=self._test,
                              interactive=self._interactive)

    def find_all(self):
        query_format = "--queryformat=[%{NAME} ]"
        all = subprocess.check_output(["rpm", "-qa", query_format]).strip()
        return all.split(" ")

    def find_with_deps(self, packages):
        result = []

        for package in packages:
            if package not in result:
                result.append(package)

            self._find_deps(package, result)

        return result

    def _find_deps(self, package, result):
        query_format = "--queryformat=[%{REQUIRENAME} ]"

        try:
            capabilities = subprocess.check_output(["rpm", "-q",
                                                    query_format,
                                                    package]).strip()
        except subprocess.CalledProcessError:
            print("Package %s not installed" % package)
            return

        filtered = [cap for cap in capabilities.split(" ")
                    if not cap.startswith("rpmlib")]

        if capabilities and filtered:
            args = ["rpm", "-q",
                    "--queryformat=[%{NAME} ]",
                    "--whatprovides"]
            args.extend(filtered)

            deps_packages = subprocess.check_output(args).strip()
            for dep_package in deps_packages.split(" "):
                if dep_package not in result:
                    result.append(dep_package)
                    self._find_deps(dep_package, result)

distro.register_package_manager("fedora", PackageManager)


class DistroInfo(interfaces.DistroInfo):
    _FEDORA_RELEASE_PATH = "/etc/fedora-release"

    def __init__(self):
        arch = self._get_architecture()

        self.name = "fedora"
        self.version = "unknown"
        self.valid = True
        self.supported = (arch in ["i386", "i686", "x86_64"])
        self.lib_dir = None

        if arch == "x86_64":
            self.lib_dir = "lib64"

        try:
            release = open(self._FEDORA_RELEASE_PATH).read().strip()
        except IOError:
            release = None
            self.valid = False

        if release == "Fedora release 18 (Spherical Cow)":
            self.version = "18"
            self.gnome_version = "3.6"
        elif release == "Fedora release 19 (Schrödinger’s Cat)":
            self.version = "19"
            self.gnome_version = "3.8"
        else:
            self.supported = False

    def _get_architecture(self):
        return subprocess.check_output(["uname", "-i"]).strip()

distro.register_distro_info(DistroInfo)



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess

from osbuild import config
from osbuild import distro
from osbuild import state
from osbuild import utils
from osbuild import xvfb

_checkers = {}


def check(remove=False, update=False, test=False, interactive=True,
          lazy=True):
    if lazy:
        if state.system_check_is_unchanged():
            return True

    package_manager = \
        distro.get_package_manager(test=test, interactive=interactive)

    distro.print_distro_info()
    packages = config.load_packages()

    checks = config.load_prerequisites()
    if not _run_checks(package_manager, checks, packages):
        return False

    xvfb_proc, orig_display = xvfb.start()

    if not _run_checks(package_manager, config.load_dependencies(), packages):
        return False

    xvfb.stop(xvfb_proc, orig_display)

    print("All the required dependencies are installed.")

    if update:
        package_manager.update()

    if remove:
        _remove_packages(package_manager, packages)

    state.system_check_touch()

    return True


def _check_binary(check):
    return subprocess.call(["which", check],
                           stdout=utils.devnull,
                           stderr=subprocess.STDOUT)

_checkers["binary"] = _check_binary


def _check_pkgconfig(check):
    return subprocess.call(["pkg-config", "--exists", check]) == 1

_checkers["pkgconfig"] = _check_pkgconfig


def _check_python(check):
    return subprocess.call(["python", "-c", check],
                           stdout=utils.devnull,
                           stderr=subprocess.STDOUT) == 1

_checkers["python"] = _check_python


def _check_gtkmodule(check):
    # Not sure we can do better than this, the gtkmodule stuff is private
    missing = True

    for libdir in config.system_lib_dirs:
        if os.path.exists("%s/gtk-2.0/modules/lib%s.so" % (libdir, check)):
            missing = False

    return missing

_checkers["gtkmodule"] = _check_gtkmodule


def _check_include(check):
    return not os.path.exists(os.path.join("/usr/include/", check))

_checkers["include"] = _check_include


def _check_dbus(check):
    found = False

    for base_dir in "system-services", "services":
        if os.path.exists(os.path.join("/usr/share/dbus-1",
                                       base_dir, "%s.service" % check)):
            found = True

    return not found

_checkers["dbus"] = _check_dbus


def _check_metacity_theme(check):
    theme = "/usr/share/themes/%s/metacity-1/metacity-theme-3.xml"
    return not os.path.exists(theme % check)

_checkers["metacity-theme"] = _check_metacity_theme


def _check_gstreamer(check, version):
    missing = True

    for libdir in config.system_lib_dirs:
        if os.path.exists("%s/gstreamer-%s/libgst%s.so" %
                          (libdir, version, check)):
            missing = False

    return missing


def _check_gstreamer_0_10(check):
    return _check_gstreamer(check, "0.10")

_checkers["gstreamer-0.10"] = _check_gstreamer_0_10


def _check_gstreamer_1_0(check):
    return _check_gstreamer(check, "1.0")

_checkers["gstreamer-1.0"] = _check_gstreamer_1_0


def _print_checks(checks):
    for check in checks:
        print("[%s] %s" % (check["checker"], check["check"]))


def _eval_check_if(check):
    if "check_if" not in check:
        return True

    distro_info = distro.get_distro_info()
    globals = {"distro": "%s-%s" % (distro_info.name, distro_info.version)}

    print(eval(check["check_if"], globals))

    return eval(check["check_if"], globals) == "True"


def _run_checks(package_manager, checks, packages):
    distro_info = distro.get_distro_info()

    failed_checks = []
    packages_not_found = []
    to_install = []

    for check in checks:
        if not _eval_check_if(check):
            continue

        checker = _checkers[check["checker"]]
        if checker(check["check"]):
            try:
                packages_for_check = packages[check["name"]][distro_info.name]
            except KeyError:
                packages_for_check = []
                packages_not_found.append(check)

            for package in packages_for_check:
                if package not in to_install:
                    to_install.append(package)

            failed_checks.append(check)

    if distro_info.supported:
        if packages_not_found:
            print("\nPackages not found for")
            _print_checks(packages_not_found)
            return False

        if to_install:
            package_manager.install_packages(to_install)
    elif failed_checks:
        print("Failed checks\n")
        _print_checks(failed_checks)

        if to_install:
            print("\nYou might try to install the following packages\n")
            print(" ".join(to_install))

        return False

    return True


def _remove_packages(package_manager, packages):
    distro_name = distro.get_distro_info().name

    to_keep = []
    for package_info in packages.values():
        if distro_name in package_info:
            for package in package_info[distro_name]:
                if package not in to_keep:
                    to_keep.append(package)

    try:
        to_keep = package_manager.find_with_deps(to_keep)
    except NotImplementedError:
        return

    all = package_manager.find_all()

    to_remove = []
    for package in all:
        if package not in to_keep:
            to_remove.append(package)

    if to_remove:
        package_manager.remove_packages(to_remove)



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import shutil

from osbuild import config
from osbuild import command


def build():
    print("\n= Building docs =\n")

    clean()

    for module in config.load_modules():
        if module.has_docs:
            print("* Generating %s" % module.name)
            os.chdir(module.get_source_dir())
            output_dir = os.path.join(config.docs_dir, module.docs_dir)
            command.run(["docker", "-I", "-c", "friendly", "-o", output_dir])

    return True

def clean():
    try:
        shutil.rmtree(os.path.join(config.docs_dir))
    except OSError:
        pass



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class PackageManager:
    """Package management"""

    def install_packages(self, packages):
        """Install packages.

           :param packages: packages to install
        """

    def remove_packages(self, packages):
        """Remove a list of packages.

           :param packages: packages to remove
        """

    def update(self):
        """Update packages to the latest version."""

    def find_all(self):
        """Return all the installed packages."""

    def find_with_deps(self, package_names):
        """Return all the installed dependencies of a list of packages.
           The packages itself are also returned in the list.

           :param packages_name: names of the packages to find
           :returns: list of packages with all the dependencies
        """


class DistroInfo:
    """Informations about the distribution"""

    def __init__(self):
        self.name = None
        """The distribution name."""

        self.version = None
        """The distribution version."""

        self.gnome_version = None
        """The major version of GNOME shipped with the distribution."""

        self.valid = False
        """If set to True we are running on this distribution and the
           attributes are all valid.
        """

        self.lib_dir = False
        """Path to the architecture specific lib directory, relative
           to the prefix.
        """

        self.supported = False
        """If set to Trye the distribution is supported."""



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess

from osbuild import command
from osbuild import config


def _chdir(func):
    def wrapped(*args, **kwargs):
        orig_cwd = os.getcwd()

        os.chdir(args[0].local)
        result = func(*args, **kwargs)
        os.chdir(orig_cwd)

        return result

    return wrapped


class Module:
    def __init__(self, path=None, name=None, remote=None,
                 branch="master", tag=None, retry=10):
        if path is None or name is None or remote is None:
            raise RuntimeError("path, name and remote are required")

        self.remote = remote
        self.local = os.path.join(path, name)
        self.tag = tag

        self._path = path
        self._name = name
        self._branch = branch
        self._retry = 10

    def _clone(self):
        os.chdir(self._path)

        command.run(["git", "clone", "--progress", self.remote, self._name],
                    retry=self._retry)

        os.chdir(self.local)

        if config.git_user_name:
            command.run(["git", "config", "user.name", config.git_user_name])

        if config.git_email:
            command.run(["git", "config", "user.email", config.git_email])

        if self.tag:
            command.run(["git", "checkout", self.tag])
        else:
            command.run(["git", "checkout", self._branch])

    def exists(self):
        return os.path.exists(os.path.join(self.local, ".git"))

    def update(self, revision=None):
        if not self.exists():
            self._clone()
            return

        orig_cwd = os.getcwd()
        os.chdir(self.local)

        if revision is None:
            if self.tag and self._head_has_tag(self.tag):
                os.chdir(orig_cwd)
                return

            revision = self.tag

        if revision == self._get_commit_id():
            os.chdir(orig_cwd)
            return

        command.run(["git", "remote", "set-url", "origin", self.remote])
        command.run(["git", "fetch"], retry=self._retry)

        if revision:
            command.run(["git", "checkout", revision])
        else:
            command.run(["git", "merge", "--ff-only",
                         "origin/%s" % self._branch])

        os.chdir(orig_cwd)

    @_chdir
    def checkout(self, revision=None):
        if revision is None:
            revision = self.tag
            if revision is None:
                revision = self._branch

        command.run(["git", "checkout", revision])

    @_chdir
    def describe(self):
        return subprocess.check_output(["git", "describe"]).strip()

    @_chdir
    def get_annotation(self, tag):
        # FIXME this is fragile, there must be a better way

        show = subprocess.check_output(["git", "show", tag])

        annotation = []
        for line in show.split("\n"):
            ignore = False
            for start in ["tag ", "Tagger: ", "Date: "]:
                if line.startswith(start):
                    ignore = True

            if line.startswith("commit "):
                break

            if not ignore:
                annotation.append(line)

        return "\n".join(annotation)

    def clean(self, new_files=False):
        try:
            os.chdir(self.local)
        except OSError:
            return False

        options = "-fd"
        if new_files:
            options = options + "x"
        else:
            options = options + "X"

        command.run(["git", "clean", options])

        return True

    def _get_commit_id(self):
        return subprocess.check_output(["git", "rev-parse", "HEAD"]).strip()

    def _head_has_tag(self, tag):
        tags = subprocess.check_output(["git", "tag", "--points-at", "HEAD"])
        return tag in tags.split("\n")


def get_module(module):
    return Module(path=config.get_source_dir(),
                  name=module.name,
                  remote=module.repo,
                  branch=module.branch,
                  tag=module.tag,
                  retry=10)



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from distutils import sysconfig
import os

from osbuild import config


def add_path(name, path):
    if not path.endswith("/"):
        path = "%s/" % path

    if name not in os.environ:
        os.environ[name] = path
        return

    splitted = os.environ[name].split(":")
    if path not in splitted:
        splitted.insert(0, path)

    os.environ[name] = ":".join(splitted)


def setup_variables():
    add_path("LD_LIBRARY_PATH", config.lib_dir)
    add_path("PATH", config.bin_dir)

    add_path("XCURSOR_PATH",
             os.path.join(config.share_dir, "icons"))
    add_path("PKG_CONFIG_PATH",
             os.path.join(config.lib_dir, "pkgconfig"))
    add_path("PYTHONPATH",
             sysconfig.get_python_lib(prefix=config.install_dir))
    add_path("PYTHONPATH",
             sysconfig.get_python_lib(prefix=config.install_dir,
                                      plat_specific=True))
    add_path("PYTHONPATH",
             os.path.dirname(os.path.dirname(__file__)))

    add_path("ACLOCAL_PATH", "/usr/share/aclocal")
    add_path("ACLOCAL_FLAGS", "-I /usr/share/aclocal")

    add_path("XDG_DATA_DIRS", "/usr/share")
    add_path("XDG_DATA_DIRS", config.share_dir)

    add_path("XDG_CONFIG_DIRS", "/etc/xdg")
    add_path("XDG_CONFIG_DIRS", os.path.join(config.etc_dir, "xdg"))

    for system_lib_dir in config.system_lib_dirs:
        modules_path = os.path.join(system_lib_dir, "gio", "modules")
        if os.path.exists(modules_path):
            add_path("GIO_EXTRA_MODULES", modules_path)

        typelib_path = os.path.join(system_lib_dir, "girepository-1.0")
        if os.path.exists(typelib_path):
            add_path("GI_TYPELIB_PATH", typelib_path)

    add_path("GI_TYPELIB_PATH",
             os.path.join(config.lib_dir, "girepository-1.0"))

    os.environ["GTK_DATA_PREFIX"] = config.install_dir
    os.environ["GTK_PATH"] = os.path.join(config.lib_dir, "gtk-2.0")
    os.environ["CC"] = "ccache gcc"
    os.environ["CXX"] = "ccache g++"

    os.environ["GCONF_DEFAULT_SOURCE_PATH"] = _get_gconf_path()
    os.environ["GCONF_SCHEMA_INSTALL_SOURCE"] = \
        "xml:merged:" + os.path.join(_get_gconf_dir(), "gconf.xml.defaults")


def setup_gconf():
    gconf_dir = _get_gconf_dir()
    gconf_path_dir = _get_gconf_path_dir()
    gconf_path = _get_gconf_path()

    if not os.path.exists(gconf_path_dir):
        os.makedirs(gconf_path_dir)

    if not os.path.exists(gconf_path):
        input = open("/etc/gconf/2/path")
        output = open(gconf_path, "w")

        for line in input.readlines():
            if "/etc/gconf" in line:
                output.write(line.replace("/etc/gconf", gconf_dir))
            output.write(line)

        output.close()
        input.close()


def _get_gconf_dir():
    return os.path.join(config.etc_dir, "gconf")


def _get_gconf_path_dir():
    return os.path.join(_get_gconf_dir(), "2")


def _get_gconf_path():
    return os.path.join(_get_gconf_path_dir(), "path.jhbuild")



# coding=utf-8

# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import subprocess

from osbuild import distro
from osbuild.plugins import interfaces
from osbuild.plugins import debian

distro.register_package_manager("ubuntu", debian.PackageManager)


class DistroInfo(interfaces.DistroInfo):
    _OS_RELEASE_PATH = "/etc/os-release"

    def __init__(self):
        arch = self._get_architecture()

        self.name = "ubuntu"
        self.version = "unknown"
        self.gnome_version = "3.4"
        self.valid = True
        self.supported = (arch in ["i386", "i686", "x86_64"])
        self.lib_dir = None

        if arch in ["i386", "i686"]:
            self.lib_dir = "lib/i386-linux-gnu"
        elif arch == "x86_64":
            self.lib_dir = "lib/x86_64-linux-gnu"

        os_info = {}

        try:
            release = open(self._OS_RELEASE_PATH).read().strip()
            for line in release.split("\n"):
                split = line.strip().split("=")
                os_info[split[0]] = split[1].replace("\"", "")
        except IOError:
            release = None
            self.valid = False

        if os_info.get("ID", None) != "ubuntu":
            self.valid = False

        self.version = os_info.get("VERSION_ID", None)

        if self.version == "13.04":
            self.gnome_version = "3.6"
        elif self.version == "13.10":
            self.gnome_version = "3.6"
        else:
            self.supported = False

    def _get_architecture(self):
        return subprocess.check_output(["uname", "-i"]).strip()

distro.register_distro_info(DistroInfo)



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import subprocess
from setuptools import setup, Extension
from distutils.cmd import Command

classifiers = ["License :: OSI Approved :: Apache Software License",
               "Programming Language :: Python :: 2",
               "Topic :: Software Development :: Build Tools"]


class LintCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        subprocess.check_call(["pep8", "."])
        subprocess.check_call(["pyflakes", "."])


setup(name="osbuild",
      packages=["osbuild", "osbuild.plugins", "osbuild"],
      version="0.17",
      description="Pull, build and test multiple source modules",
      author="Daniel Narvaez",
      author_email="dwnarvaez@gmail.com",
      url="http://github.com/dnarvaez/osbuild",
      classifiers=classifiers,
      test_suite="osbuild.tests",
      cmdclass={"lint": LintCommand},
      ext_modules=[Extension("osbuild.sourcestamp", ["src/sourcestamp.c"])])



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import signal
import string
import random
import subprocess

from osbuild import config
from osbuild import command


def run(cmd):
    args = [cmd, "--home-dir", config.home_dir]

    resolution = config.get_pref("RESOLUTION")
    if resolution:
        args.extend(["--resolution", resolution])

    output = config.get_pref("OUTPUT")
    if output:
        args.extend(["--output", output])

    signal.signal(signal.SIGINT, signal.SIG_IGN)

    os.environ["SUGAR_DEVELOPER"] = "1"

    print "Type Shift-Alt-Q inside sugar to close."
    command.run(args)


def run_test(test_cmd, test_path):
    args = [test_cmd,
            "--test-command", "python -u %s" % test_path,
            "--virtual"]

    with open(config.log_path, "a") as f:
        result = subprocess.call(args, stdout=f, stderr=subprocess.STDOUT)
        return result == 0


def collect_logs(source_path):
    logs = {}
    for filename in os.listdir(source_path):
        if filename.endswith(".log"):
            path = os.path.join(source_path, filename)
            with open(path) as f:
                logs[filename] = f.read()

    with open(config.log_path, "a") as f:
        for filename, log in logs.items():
            f.write("\n===== %s =====\n\n%s" % (filename, log))


def _get_random_id():
    return ''.join(random.choice(string.letters) for i in range(8))



# coding=utf-8

# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from osbuild import distro
from osbuild.plugins import interfaces


class PackageManager(interfaces.PackageManager):
    def __init__(self, test=False, interactive=True):
        pass

    def install_packages(self, packages):
        pass

    def remove_packages(self, packages):
        pass

    def update(self):
        pass

    def find_all(self):
        return []

    def find_with_deps(self, packages):
        return []

distro.register_package_manager("unknown", PackageManager)


class DistroInfo(interfaces.DistroInfo):
    def __init__(self):
        self.lib_dir = None
        self.name = "unknown"
        self.version = "unknown"
        self.gnome_version = "3.4"
        self.valid = True
        self.supported = False

distro.register_distro_info(DistroInfo)



# Copyright 2013 Daniel Narvaez
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

_package_managers = {}
_supported_distros = []
_distro_info = None


def register_distro_info(distro_info):
    global _supported_distros
    _supported_distros.append(distro_info)


def register_package_manager(name, package_manager):
    global _package_managers
    _package_managers[name] = package_manager


def get_package_manager(test=False, interactive=True):
    global _package_managers
    package_manager_class = _package_managers[get_distro_info().name]
    return package_manager_class(test=test, interactive=interactive)


def print_distro_info():
    info = get_distro_info()
    print("\n= Distribution information =\n")
    print("Name: %s" % info.name)
    print("Version: %s" % info.version)
    print("GNOME version: %s" % info.gnome_version)
    print("Lib directory: %s" % info.lib_dir)
    print("Supported: %s\n" % info.supported)


def get_distro_info():
    global _supported_distros
    global _distro_info

    if _distro_info is not None:
        return _distro_info

    unknown_distro = None

    for info_class in _supported_distros:
        info = info_class()
        if info.name == "unknown":
            unknown_distro = info
        elif info.valid:
            _distro_info = info

    if _distro_info is None:
        _distro_info = unknown_distro

    if not _distro_info.supported:
        print("*********************************************************\n"
              "You are running an unsupported distribution. You might be\n"
              "able to make sugar work by installing or building \n"
              "packages but it certainly won't work out of the box.\n"
              "You are strongly encouraged to pick one of the supported \n"
              "distributions listed in the README.\n"
              "*********************************************************\n")

    return _distro_info
